#pragma once

#include "PGJsonResource.h"
#include "PGJsonCommand.h"
#include "PGClientSocket.h"
#include "PGJsonParser.h"

//-----------------------------------------------------------------------------
/**
@brief イベントハンドラ基本クラス

イベントドリブンの枠組みを提供します。
ユーザーの操作により発生したイベントは、ダイアログ毎に振り分けられ、対応するオブジェクトに通知されます。
アプリケーション開発者は、ダイアログ毎に、このクラスを継承した実装クラスを作成して下さい。
*/
class PGDialog
{
  public:
	PGDialog();
	virtual ~PGDialog();
	
	// ユーザが継承する関数
	virtual void OnInit();
	virtual void OnChangeBefore( PGClientSocket *pgsock, PGJsonCommand *json );
	virtual void OnChange( PGClientSocket *pgsock, PGJsonCommand *json );
	virtual void OnCondition( PGClientSocket *pgsock, PGJsonCommand *json );
	virtual void OnUpdate(PGJsonObject* to ,PGJsonObject* from);
	virtual void OnTouch( PGClientSocket *pgsock, PGJsonObject *jobj ,const SGString actiontype );
	virtual void OnTimer(TimeVal time,unsigned int id);
	virtual void OnFinal();

	// ユーザが継承しない関数
	void Init( const SGString &dialogname, const char *filename );

	PGJsonResource& GetResourceUnique( const SGString& sessionid);
	PGJsonResource& GetResourceShare();

	void UpdateDialog( PGClientSocket *pgsock, PGJsonCommand *jo, PGJsonResource* resource );
	void UpdateResourceUnique( PGJsonCommand *jo, const SGString& sessionid );
	void UpdateResourceShare( PGJsonCommand *jo );
  private:
	void UpdateResource( PGJsonCommand *jo, PGJsonResource* resource );

  public:
	bool SendUpdateShare(bool allflag=false);
	bool SendUpdateUnique(const SGString& sessionid, bool allflag=false);

	bool LoadJsonShare();
	bool LoadJsonUnique( const SGString& sessionid );
  private:
	void LoadJson( const SGString &filename, const SGString& sessionid );
  public:


	int AddSession( const SGString &sessionid );
	int RemoveSession( const SGString &sessionid );

	void SetTimerInterval(int time,unsigned int id);
	void SetSingleTimerInterval(int time,unsigned int id);

	typedef std::map<SGString,PGJsonResource> MapList;               ///< @brief セッションIDとダイアログオブジェクトの連想配列
	typedef std::map<SGString,PGJsonResource>::iterator MapIterator; ///< @brief @ref MapList のイテレーター

  protected:
	SGString mDialogName;       ///< @brief カレントダイアログ名
	SGString mFileName;         ///< @brief ファイル名
	MapList mSessionResourceList; ///< @brief セッション毎ダイアログのリソースリスト
};

//-----------------------------------------------------------------------------
/**
@brief イベントハンドラリスト
*/
class PGDialogList
{
  public:
//  virtual PGDialog *Get( const SGString &id, const SGString &name );
	
	bool Has( const SGString &name );
	PGDialog *Search( const SGString &name );
	
	void Add( const SGString &name, PGDialog *dialog );
	void Remove( const SGString &name );
	
	void RemoveBySession( const SGString &sessionid );

  private:
	typedef std::map<SGString,PGDialog*> List;               ///< @brief イベントハンドラの可変長配列
	typedef std::map<SGString,PGDialog*>::iterator Iterator; ///< @brief @ref List のイテレーター
	List mDialogList; ///< @brief イベントハンドラリスト
};

