#pragma once
#include <iostream>
#include <set>
#include "JsonParser.h"
#include "PGValue.h"

//-----------------------------------------------------------------------------
/**
@brief JSON オブジェクト

JSON のオブジェクト型データを保持します。
*/
class PGJsonObject
{
  public:
	PGJsonObject(PGJsonObject* parent);
	virtual ~PGJsonObject();

	// 既に仮想化を利用しているもの
	virtual bool HasGroup(); // const
	virtual bool AddObject( PGJsonObject *jo );
	virtual ostream &OutJson( ostream &out ); // const
	virtual ostream &OutJson( ostream &out, int &b ); // const
	virtual ostream &OutJsonMember( ostream &out, int &b ); // const
	virtual ostream &OutJsonDirtyMember( ostream &out, int &b );
	virtual ostream &OutJsonList( ostream &out, int &b ); // const
	virtual PGJsonObject* SearchName( const SGString &name ); // const
	virtual PGJsonObject* SetValue( const SGString &key, PGValue value );

	// まだ仮想化として利用されていない
	virtual bool Parse( JsonParser &jp );
	virtual SGString GetJson(); // const
	virtual PGJsonObject* SetValueByName( const SGString &key, PGValue value, const SGString &name );
	virtual PGValue& GetValue( const SGString &key ); // const
	virtual PGJsonObject* GetParent();
	virtual bool HasKey( const SGString &key ) const;

	void SetDirty(const SGString &key );
	void ResetDirty();
	bool IsDirty();

	typedef vector<PGJsonObject*> JObjectList;                 ///< @brief JSON オブジェクトリスト型
	typedef vector<PGJsonObject*>::iterator JOIterator;   ///< @brief @ref List のイテレーター型
	typedef map<SGString,PGValue> KeyValueList;         ///< @brief キー値リスト型
	typedef map<SGString,PGValue>::iterator KVIterator; ///< @brief @ref KeyValueList のイテレーター型
	typedef set<SGString> DirtyList;         ///< @brief キー値リスト型
	typedef set<SGString>::iterator DLIterator; ///< @brief @ref KeyValueList のイテレーター型

	KVIterator KeyValueBegin();
	KVIterator KeyValueEnd();

	SGString mName;     ///< @brief 名前
private:
	PGJsonObject* mParent;
	DirtyList mDirty;        ///< @brief ダーティフラグ
	KeyValueList mList; ///< @brief リスト
};
//-----------------------------------------------------------------------------
/**
@brief ラジオボタン オブジェクト
*/
class PGJsonRadioButtonObject : public PGJsonObject
{
  public:
	PGJsonRadioButtonObject(PGJsonObject* parent );
	virtual ~PGJsonRadioButtonObject();

	virtual PGJsonObject* SetValue( const SGString &key, PGValue value );
};


//-----------------------------------------------------------------------------
/**
@brief リスト付 JSON オブジェクト
*/
class PGJsonObjectHasList : public PGJsonObject
{
  public:
	PGJsonObjectHasList(const char* listkey, PGJsonObject* parent );
	virtual ~PGJsonObjectHasList();

	virtual bool HasGroup();
	virtual bool AddObject( PGJsonObject *jo );
	virtual ostream &OutJsonList( ostream &out, int &b );
	virtual PGJsonObject* SearchName( const SGString &name );

	SGString mListKey; ///< @brief リストのキー名
	JObjectList mObjectList;  ///< @brief 子 JSON オブジェクトリスト
};

//-----------------------------------------------------------------------------
/**
@brief ラジオボタン グループオブジェクト
*/
class PGJsonRadioButtonGroup : public PGJsonObjectHasList
{
  public:
	PGJsonRadioButtonGroup(PGJsonObject* parent );
	virtual ~PGJsonRadioButtonGroup();

	void SwitchOff();
};
